@@ -47,7 +47,7 @@ def test_push_devpi(loghub, monkeypatch, mock_http_api):
 def test_main_push_pypi(monkeypatch, tmpdir):
     from devpi.push import main
     l = []
-    def mypost(method, url, data, headers, auth=None):
+    def mypost(method, url, data, headers, auth=None, cert=None):
         l.append((method, url, data))
         class r:
             status_code = 201
